# Generated by Django 2.2.28 on 2024-04-24 05:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0005_auto_20240207_0630'),
    ]

    operations = [
        migrations.AddField(
            model_name='paypalwebhook',
            name='auth_hash',
            field=models.CharField(default='', max_length=255, verbose_name='Auth Hash'),
        ),
        migrations.AlterField(
            model_name='paypalwebhook',
            name='url',
            field=models.CharField(max_length=255, verbose_name='URL'),
        ),
        migrations.AlterUniqueTogether(
            name='paypalwebhook',
            unique_together={('url', 'auth_hash')},
        ),
    ]
