# -*- coding: utf-8 -*-
# Generated by Django 1.11.27 on 2020-01-24 14:28
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('django_paypal', '0002_auto_20171002_1551'),
    ]

    operations = [
        migrations.AddField(
            model_name='paypalpayment',
            name='initial_response_object',
            field=models.TextField(blank=True, null=True, verbose_name='initial post response'),
        ),
        migrations.AddField(
            model_name='paypalpayment',
            name='related_resource_id',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='related resource id'),
        ),
        migrations.AddField(
            model_name='paypalpayment',
            name='update_response_object',
            field=models.TextField(blank=True, null=True, verbose_name='updated get response'),
        ),
        migrations.AlterField(
            model_name='paypalitem',
            name='transaction',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='items', to='django_paypal.PaypalTransaction', verbose_name='transaction'),
        ),
        migrations.AlterField(
            model_name='paypaltransaction',
            name='payment',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='transactions', to='django_paypal.PaypalPayment', verbose_name='payment'),
        ),
    ]
